/**********************************************************************/
/** This file contains the code to compute DD estimates using LLR    **/
/**    and run the estimation procedure over the Monte-Carlo         **/
/**    datasets given the master data, information about policy      **/
/**    regime (education subsidy available or not) and selection     **/
/**    mechanism. Uses Gaussian or Epanechnikov kernel weights.      **/
/**    Routines in here are called by the main program, CF.do.       **/
/**                                                                  **/
/** Monica Costa Dias                                                **/
/** September 2008                                                   **/
/**********************************************************************/



******************************************************
** DD estimation procedure - unconditional on X     **
**                                                  **
**   need to know: observed outcome                 **
**   input as global: education status ($d)         **
**                    test score ($s)               **
******************************************************

cap program drop DD
program define DD

	** Preparing variables and weights **
	*************************************
	qui gen elig = ($s>=4)                           /* eligibility status                   */
	qui gen cs   = ($s-4)/$h                         /* centered regressor (score)           */
*	qui gen k    = (abs(cs)<=1)*3*(1-(cs^2))/4       /* Choose: Epanechnikov kernel weights  */
	qui gen k    = exp(-cs*cs/2)/sqrt(2*3.141593)    /*         or Gaussian kernel weights   */


	** Local linear regressions **
	******************************
	qui reg y cs [iw=k] if elig==0                   /* expected outcome among ineligibles   */
	matrix e = e(b)
	scalar y0hat = e[1,2]

	qui reg $d cs [iw=k] if elig==0                  /* participation rate among ineligibles */
	matrix e = e(b)
	scalar p0hat = e[1,2]

	qui reg y cs [iw=k] if elig==1                   /* expected outcome among eligibles     */
	matrix e = e(b)
	scalar y1hat = e[1,2]

	qui reg $d cs [iw=k] if elig==1                  /* participation rate among eligibles   */
	matrix e = e(b)
	scalar p1hat = e[1,2]


	** Form DD estimator **
	***********************
	global DD = (y1hat-y0hat)/(p1hat-p0hat)          /* DD estimator                         */


	** Cleaning dataset **
	**********************
	drop y elig cs k

end



*************************************************************
** Monte Carlo replications of the DD estimation procedure **
*************************************************************

cap program drop MCDD
program define MCDD

	** Monte-Carlo replications **
	******************************
	forvalues i=1/200 {

		display "MC replication ", `i'

		local lb = (`i'-1)*$N+1                   /* read data              */
		local ub = `i'*$N
		qui use $dtafile in `lb'/`ub', clear

		qui gen y    = $d*ln(y1)+(1-$d)*ln(y0)    /* observed outcome       */
		qui DD                                    /* DD estimator           */
		global DD`i' = $DD

	}


	** Storing estimates **
	***********************
	drop _all
	set obs 200
	gen trial = _n
	gen DD    = .
	forvalues i=1/200 {
		qui replace DD=${DD`i'} if trial == `i'
	}

end




