/**********************************************************************/
/** This file contains the code to compute PS matching estimates     **/
/**    and run the estimation procedure over the Monte-Carlo         **/
/**    datasets given the master data, selection mechanism,          **/
/**    information about policy regime (education subsidy available  **/
/**    or not) and the set of selected matching variables. Uses      **/
/**    Epanechnikov kernel weights and implemets the estimation      **/
/**    routines for both the ATT and ATNT. Routines in here are      **/
/**    called by the main program, matching.do.                      **/
/**                                                                  **/
/** Monica Costa Dias                                                **/
/** September 2008                                                   **/
/**********************************************************************/



*********************************************************************
** PS matching estimation procedure: estimates ATT and ATNT over   **
**   the common support for p(X)                                   **
**                                                                 **
**   need to know: observed outcome (y)                            **
**   input as global: education status ($d)                        **
**                    matching variables ($matchX)                 **
*********************************************************************

cap program drop matching
program define matching

	qui probit $d $matchX
	qui predict p, p


	** Estimating the ATT using PS matching **
	******************************************

	qui gen group = $d                /* setting group=1 to be the educated (treated)                    */
	qui Kernel1D                      /* matching observations in group=0 to each observation in group=1 */

	qui sum y [iw=w] if group==0      /* mean outcome among the non-treated matched to the treated and   */
	local Ey0 = r(mean)               /*   weighted by the distribution of ps among treated              */
	qui sum y if group==1 & w~=0      /* mean outcome among the treated for whom at least one similar    */
	local Ey1 = r(mean)               /*   non-treated is found                                          */

	scalar matchATT = `Ey1'-`Ey0'     /* PS matching estimate of the ATT                                 */


	** Estimating the ATNT using PS matching **
	*******************************************

	qui replace group = 1-$d          /* setting group=1 to be the non-educated (non-treated)            */
	qui Kernel1D                      /* matching observations in group=0 to each observation in group=1 */

	qui sum y [iw=w] if group==0      /* mean outcome among the treated matched to the non-treated and   */
	local Ey1 = r(mean)               /*   weighted by the distribution of ps among non-treated          */
	qui sum y if group==1 & w~=0      /* mean outcome among the non-treated for whom at least one        */
	local Ey0 = r(mean)               /*   similar treated is found                                      */

	scalar matchATNT = `Ey1'-`Ey0'    /* PS matching estimate of the ATNT                                */

end



*********************************************************************
** 1 dimensional PS matching procedure using Epanechnikov kernel   **
**   weights with a constant bandwidth.                            **
**                                                                 **
**   need to know: propensity score (p)                            **
**                 observed outcome (y)                            **
**                 group to be matched (group)                     **
**   input as global: bandwidth ($h)                               **
**                    step to form grid in p ($step)               **
**                    total numbe of observations in dataset ($N)  **
*********************************************************************

cap program drop Kernel1D
program define Kernel1D

	** generate working variables **
	********************************
	cap qui drop w
	qui gen w    = 0                                   /* variable to contain matching weights for each observation in group=1 */
	qui gen k    = .                                   /* variable to contain final kernel weights                             */
	qui gen dist = .                                   /* variable to contain standardised propensity score                    */


	** distribution of dummy variable group **
	******************************************
	qui sort group
	qui count if group==0                              /* number of observations in group=0                                    */
	local i1=r(N)+1                                    /* first observation in group=1                                         */


	** matching weights for both groups (0 and 1) **
	************************************************
	forvalues i=`i1'/$N {                                            /* Matching each observation in group=1                   */

		qui replace dist = abs(p-p[`i'])/$h if group==0           /* construct standartized distance                        */
		qui replace k    = (dist<1)*(1-dist^2)*3/4 if group==0    /* and compute kernel weights among group=0               */

		qui sum k if group==0                                     /* check whether there is at least 1 matching observation */
		if r(mean)==0 {                                           /*    if not: matching weight is zero for observation i   */
			qui replace w = 0 if _n==`i'
		}
		else {                                                    /*    if yes: positive weight for observation i and       */
			qui replace w = w+ k/(r(mean)*r(N)) if group==0    /*            matching observations                       */
			qui replace w = 1 if _n==`i'
		}
	}

      
	** cleaning dataset **
	**********************
	qui drop dist k

end



**********************************************************************
** Monte Carlo replications of the PS matching estimation procedure **
**********************************************************************

cap program drop MCmatching
program define MCmatching


	** Monte-Carlo replications **
	******************************
	forvalues i=1/200 {

		display "MC replication ", `i'

		local lb = (`i'-1)*$N+1                        /* read data              */
		local ub = `i'*$N
		qui use $dtafile in `lb'/`ub', clear

		qui gen y = $d*ln(y1)+(1-$d)*ln(y0)            /* observed outcome       */
		qui matching
		global matchATT`i'  = matchATT
		global matchATNT`i' = matchATNT

	}


	** Storing estimates **
	***********************
	drop _all
	set obs 200
	gen trial     = _n
	gen matchATT  = .
	gen matchATNT = .
	forvalues i=1/200 {
		qui replace matchATT  = ${matchATT`i'} if trial == `i'
		qui replace matchATNT = ${matchATNT`i'} if trial == `i'
	}

end
